package ru.dbotthepony.mc.otm.capability;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.Tag;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.common.util.INBTSerializable;
import net.minecraftforge.registries.RegistryManager;

import javax.annotation.Nullable;
import java.util.Objects;
import java.util.UUID;

public record MatterTask(UUID id, Item item, int in_progress, int finished, int required) {
	public MatterTask(UUID id, Item item, int in_progress, int finished, int required) {
		this.id = id;
		this.item = item;
		this.in_progress = Math.max(0, in_progress);
		this.finished = Math.max(0, finished);
		this.required = Math.max(0, required);
	}

	public ItemStack stack() {
		return new ItemStack(item, 1);
	}

	public ItemStack stack(int amount) {
		return new ItemStack(item, amount);
	}

	public int total() {
		return in_progress + finished + required;
	}

	public MatterTask shrinkRequired(int amount) {
		return new MatterTask(id, item, in_progress + amount, finished, required - amount);
	}

	public MatterTask shrinkInProgress(int amount) {
		return new MatterTask(id, item, in_progress - amount, finished + amount, required);
	}

	public CompoundTag serializeNBT() {
		CompoundTag tag = new CompoundTag();

		tag.putLong("id_l", id.getLeastSignificantBits());
		tag.putLong("id_u", id.getMostSignificantBits());
		tag.putString("item", Objects.requireNonNull(item.getRegistryName()).toString());
		tag.putInt("in_progress", in_progress);
		tag.putInt("finished", finished);
		tag.putInt("required", required);

		return tag;
	}

	@Nullable
	public static MatterTask deserializeNBT(@Nullable Tag nbt) {
		if (nbt == null)
			return null;

		if (nbt instanceof CompoundTag tag) {
			Item get_item = RegistryManager.ACTIVE.getRegistry(Item.class).getValue(new ResourceLocation(tag.getString("item")));

			if (get_item != null)
				return new MatterTask(
					new UUID(tag.getLong("id_u"), tag.getLong("id_l")),
					get_item,
					tag.getInt("in_progress"),
					tag.getInt("finished"),
					tag.getInt("required")
				);
		}

		return null;
	}
}
